//=============================================================================
// Copyright  Micromega Corporation 2005-2008. All rights reserved.
//
// @file	graph.c
// @target	Atmel WinAVR C compiler
//
// This program demonstrates the use of the uM-FPU V3 floating point
// unit with the Atmel Acr using an SPI interface. It draws a simple
// graph of the sine and cosine functions in the terminal window.
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	February 1, 2008
//	- fixed second graph to be cosine
//	September 20, 2007
//	- original version
//=============================================================================

#include <avr/io.h>
#include <stdio.h>
#include <math.h>
#include "uart.h"
#include "fpu.h"
#include <util/delay.h>

FILE uart_stream = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

//-------------------- uM-FPU register definitions ----------------------------

#define Angle		1				// angle
#define Point		2				// point on graph

//------------------ local prototypes -----------------------------------------

void graph(void);

//------------------ main -----------------------------------------------------

int main(void) {
	int i;

	uart_init();					// initialize the UART
	stdout = &uart_stream;			// set stdout to UART

	printf("\r\nGraph demo\r\n");	// program name

	// reset the uM-FPU and print version
	if (fpu_reset() == SYNC_CHAR) {	// reset FPU and check synchronization
		print_version();			// print version
		print_CRLF();
	}
	else {
		printf("uM-FPU not detected");
		return(0);
	}

	// graph sine from 0 to 360 degrees (2*PI radians)
	printf("\r\nGraph of sine\r\n");
	printf("-1        0         1\r\n");
	printf("---------------------\r\n");
    for (i = 0; i <= 20; i++) {
		fpu_write4(SELECTA, Angle, LOADPI, FSET0);
		fpu_write4(FMULI, 2*i, FDIVI, 20);
		fpu_write(SIN);
		graph();
	}

    // graph cosine from 0 to 360 degrees (2*PI radians)
	printf("\r\nGraph of cosine\r\n");
	printf("-1        0         1\r\n");
	printf("---------------------\r\n");
    for (i = 0; i <= 20; i++) {
		fpu_write4(SELECTA, Angle, LOADPI, FSET0);
		fpu_write4(FMULI, 2*i, FDIVI, 20);
		fpu_write(COS);
		graph();
	}

	printf("\r\nDone.\r\n");
	return 0;
}

//-------------------- graph --------------------------------------------------

void graph(void) {
	int i;
	char point, ch;

    // calculate point on graph for value
	fpu_write4(SELECTA, Point, FSET, Angle);
	fpu_write4(FMULI, 10, ROUND, FIX);
	fpu_wait();
	fpu_write(LREADBYTE);
	fpu_readDelay();
    point = fpu_read();

    // plot the point on the graph
	for (i = -10; i <= 10; i++) {
		if (i == point) ch = '*';
		else if (i == 0) ch = '|';
		else ch = ' ';
		putchar(ch);
	}

    // display the angle value
	putchar(' ');
	fpu_write2(SELECTA, Angle);
	print_float(85);
	print_CRLF();
}
