//=============================================================================
// Copyright  Micromega Corporation 2005-2008. All rights reserved.
//
// @file	sample.c
// @target	Atmel WinAVR C compiler
//
// Test routine for libm_fpu
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	August 31, 2007
//	- updated for uM-FPU V3.1
//	November 17, 2006
//	- original version
//=============================================================================

#include <avr/io.h>
#include <stdio.h>
#include <math.h>
#include "uart.h"
#include "fpu.h"
#include <util/delay.h>

// bit set and clear macros
#define bit_set(sfr, bit) (_SFR_BYTE(sfr) = _SFR_BYTE(sfr) | _BV(bit))
#define bit_clr(sfr, bit) (_SFR_BYTE(sfr) = _SFR_BYTE(sfr) & ~_BV(bit))

FILE uart_stream = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

//-------------------- uM-FPU register definitions ----------------------------

#define DiameterIn		1				// diameter in inches
#define Circumference	2				// circumference
#define Area			3				// area
#define Pi				4				// constant pi

//------------------ main -----------------------------------------------------

int main(void) {
	int diameterCm;

	// initialize the UART and set as stdout
	uart_init();
	stdout = &uart_stream;

	// print program name
	printf("\r\n\r\nConversion Example\r\n------------------\r\n");

	//------------------------------------
	// Reset the uM-FPU and print version.
	//------------------------------------
	if (fpu_reset() == SYNC_CHAR) {
		print_version();
		print_CRLF();
		print_CRLF();
	}
	else {
		printf("uM-FPU not detected");
		return(0);
	}

	//-----------------------------
	// Load constant for later use.
	//-----------------------------
	fpu_write4(SELECTA, Pi, LOADPI, FSET0);

	//---------------------------------------------------------------------------
	// Get diameter in centimeters.  The value would typically come from a sensor
	// reading, but in this example an assumed value of 25 is used.
	//---------------------------------------------------------------------------
	diameterCm = 25;
	printf("Diameter (cm):       %i\r\n", diameterCm);

	//------------------------------
	// convert inches to centimeters
	//------------------------------
	fpu_write4(SELECTA, DiameterIn, FSETI, diameterCm);
	fpu_write2(FCNV, 5);
	printf("Diameter (in.):      ");
	print_float(0);
	print_CRLF();

	//------------------------------
	// circumference = diameter * pi
	//------------------------------
	fpu_write4(SELECTA, Circumference, FSET, DiameterIn);
	fpu_write2(FMUL, Pi);
	printf("Circumference (in.): ");
	print_float(0);
	print_CRLF();

	//----------------------------
	// area = (diameter / 2)^2 * pi
 	//----------------------------
	fpu_write4(SELECTA, Area, FSET, DiameterIn);
	fpu_write2(FDIVI, 2);
	fpu_write4(FMUL, Area, FMUL, Pi);
	printf("Area (sq.in.):       ");
	print_float(0);
	print_CRLF();

	printf("\r\nDone.\r\n");
	return 0;
}
