//=============================================================================
// Copyright  Micromega Corporation 2005-2007. All rights reserved.
//
// @file	template.c
// @target	Atmel WinAVR C compiler
//
// This program provides an example of using the uM-FPU V3.1 floating point
// coprocessor with the WinAVR compiler using an SPI interface. It shows
// the initial FPU reset sequence and prints the version number of the FPU chip.
//
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	October 15, 2007
//	- original version
//=============================================================================

#include <avr/io.h>
#include <stdio.h>
#include "uart.h"
#include "fpu.h"

FILE uart_stream = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

//------------------ local prototypes -----------------------------------------

//------------------ uM-FPU register definitions ------------------------------

//------------------ main -----------------------------------------------------

int main(void) {

	// initialize the UART and set as stdout
	uart_init();
	stdout = &uart_stream;

	// print program name
	printf("\r\n\r\nWinAVR template\r\n");

	// Reset the uM-FPU and print version
	if (fpu_reset() == SYNC_CHAR) {
		print_version();
		print_CRLF();
	}
	else {
		printf("uM-FPU not detected");
		return(0);
	}

//	Insert main application code here.
//	FPU instructions are sent using fpu_write routines.
//	e.g.
//		fpu_write4(SELECTA, 1, LOADPI, FSET0);

	printf("\r\nDone.\r\n");
	return 0;
}
